if (typeof(simplemail) == "undefined")
  var simplemail = {a: "tree.js"};

simplemail.DefaultTreeView = function() {
  var pub = {};

  pub.DROP_ON     = Components.interfaces.nsITreeView.DROP_ON,
  pub.DROP_BEFORE = Components.interfaces.nsITreeView.DROP_BEFORE,
  pub.DROP_AFTER  = Components.interfaces.nsITreeView.DROP_AFTER,

  pub.setTree = function(treebox) {
    this.treebox = treebox;
    if (treebox) 
      this.tree = treebox.treeBody.parentNode;
  },
  pub.isContainer = function(row) { return false; },
  pub.isSeparator = function(row) { return false; },
  pub.isSorted = function() { return false; },
  pub.cycleHeader = function(column) {},
  pub.cycleCell = function(row, column) { this.tree.view.selection.select(row); },
  pub.getLevel = function(row) { return 0; },
  pub.getImageSrc = function(row, column) { return null; },
  pub.getRowProperties = function(row, props) {},
  pub.getCellProperties = function(row, column, props) {},
  pub.getColumnProperties = function(column, props) {},
  pub.canDrop = function(row, orientation) { return false; },

  pub.extend = function(treeView) {
    for(var i in this) {
      if (treeView[i] == null) treeView[i] = this[i];
    }
  },

  pub.atomService = Components.classes["@mozilla.org/atom-service;1"].getService(Components.interfaces.nsIAtomService),

  pub.nextDirection = {
    ""           : "ascending",
    "ascending"  : "descending",
    "descending" : ""
  },

  pub.getSortedColumn = function() {
    var column = this.tree.columns.getSortedColumn();
    if (column) return simplemail.SimpleMailDom.get(column.id);
  },

  pub.getCheckboxColumn = function() {
    for(var i = 0; i < this.tree.columns.count; i++) {
      var column = this.tree.columns.getColumnAt(i);
      if (column.type == column.TYPE_CHECKBOX) {
        return column;
      }
    }
  },

  pub.forEachChecked = function(onItem) {
    var column = this.getCheckboxColumn();
    if (!column) return;

    var found;
    this.treebox.beginUpdateBatch();
    for (var i = this.rowCount - 1; i >= 0; i--) {
      if (this.getCellValue(i, column) == "true") {
        found = true;
        onItem(i);
      }
    }
    this.treebox.endUpdateBatch();
    return found;
  },

  pub.forEachSelected = function(onItem) {
    var start = new Object();
    var end = new Object();
    var rangeCount = this.tree.view.selection.getRangeCount();

    var found;
    this.treebox.beginUpdateBatch();
    for (var i = rangeCount - 1; i >= 0; i--){
      this.tree.view.selection.getRangeAt(i, start, end);
      for (var j = end.value; j >= start.value; j--) {
        found = true;
        onItem(j);
      }
    }
    this.treebox.endUpdateBatch();
    return found;
  },

  // Event handling 
  pub.onDblClick = function(event) {
    var row = {}, column = {}, child = {};
    this.treebox.getCellAt(event.clientX, event.clientY, row, column, child);
    if (row.value != -1) {
      event.preventDefault();
      this.action();
    }
  },
  
  // userok: fix error about undeclared class function "getParentIndex"
  pub.getParentIndex = function(c) {
    //console.error("SM.getParentIndex", c);//userok_log
    return -1;
  },
  
  pub.select = function() {
    this.tree.view.selection.rangedSelect(this.tree.currentIndex, this.tree.currentIndex, true);
    if (this.tree.currentIndex < this.rowCount - 1) {
      this.tree.currentIndex++;
      this.tree.onselect();
    }
  },
  
  pub.selectAll = function() {
    this.tree.view.selection.selectAll();
  }
  
  // userok: rewrited
  pub.onKeyPress = function(event) {
    //console.error("SM.tree_onKeyPress", event, this.treebox);//userok_log
    if (!(event.ctrlKey || event.metaKey || event.altKey)) {
      switch (event.keyCode) {
        // delete
        case event.DOM_VK_DELETE:
        case event.DOM_VK_BACK_SPACE:
          event.preventDefault(); 
          return this.delete(event.shiftKey);
        // action
        case event.DOM_VK_RETURN:
          event.preventDefault();
          return this.action();
        // select - userok: what is strange select?
        case event.DOM_VK_INSERT:
          event.preventDefault();
          return this.select();
        // select all by *
        case 0:
          if (event.key == '*' || event.charCode == 42) {
            event.preventDefault();
            return this.selectAll();
          }
      }
    }
    // select all by ctrl+a
    if ((event.ctrlKey || event.metaKey) && !(event.altKey || event.shiftKey) && (event.key == 'a' || event.charCode == 97)) {
      event.preventDefault();
      this.selectAll();                                                           
    }
  },

  pub.action = function() {},
  pub.delete = function() {}
  
  return pub;
}();