Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
//Components.utils.import(InJlc291cmNlOi8vZ3JlL21vZHVsZXMvZGV2dG9vbHMvQ29uc29sZS5qc20i); // userok_log

var Ci          = Components.interfaces;
var CLASS_ID    = Components.ID("{6224daa1-71a2-4d1a-ad90-01ca1c08e323}");
var CLASS_NAME  = "Simple AutoComplete";
var CONTRACT_ID = "@mozilla.org/autocomplete/search;1?name=simple-autocomplete";

// Implements nsIAutoCompleteResult
function SimpleAutoCompleteResult(searchString, searchResult,
                                  defaultIndex, errorDescription,
                                  results, storage) {
  this._searchString = searchString;
  this._searchResult = searchResult;
  this._defaultIndex = defaultIndex;
  this._errorDescription = errorDescription;
  this._results = results;
  this._storage = storage;
}

SimpleAutoCompleteResult.prototype = {
  _searchString: "",
  _searchResult: 0,
  _defaultIndex: 0,
  _errorDescription: "",
  _results: [],

  /**
   * The original search string
   */
  get searchString() {
    return this._searchString;
  },

  /**
   * The result code of this result object, either:
   *         RESULT_IGNORED   (invalid searchString)
   *         RESULT_FAILURE   (failure)
   *         RESULT_NOMATCH   (no matches found)
   *         RESULT_SUCCESS   (matches found)
   */
  get searchResult() {
    return this._searchResult;
  },

  /**
   * Index of the default item that should be entered if none is selected
   */
  get defaultIndex() {
    return this._defaultIndex;
  },

  /**
   * A string describing the cause of a search failure
   */
  get errorDescription() {
    return this._errorDescription;
  },

  /**
   * The number of matches
   */
  get matchCount() {
    return this._results.length;
  },

  /**
   * Get the value of the result at the given index
   */
  getValueAt: function(index) {
    return this._results[index];
  },
  
  getFinalCompleteValueAt: function (index) {
  return this.getValueAt(index);
  },

  /**
   * Get the comment of the result at the given index
   */
  getCommentAt: function(index) {
    return null;
  },

  /**
   * Get the style hint for the result at the given index
   */
  getStyleAt: function(index) {
    return "";
  },

  getImageAt: function(index) {
    return null;
  },

  getLabelAt: function(index) {
    return this._results[index];
  },

  /**
   * Remove the value at the given index from the autocomplete results.
   * If removeFromDb is set to true, the value should be removed from
   * persistent storage as well.
   */
  removeValueAt: function(index, removeFromDb) {
    if (removeFromDb) {
      this._storage.deleteAddress(this._results[index].id); // userok: async
    }
    this._results.splice(index, 1);
  },

  QueryInterface: XPCOMUtils.generateQI([ Ci.nsIAutoCompleteResult ])
};

function Address(id, name, email) {
  this.id = id;
  this.name = name;
  this.email = email;

  this.toString = function() {
    return this.name + (this.name && this.email ? " " : "") + (this.email ? "<" + this.email + ">" : "");
  }
}

function Storage() {
  var file = getDataDirectory();
  file.append("simplemail.sqlite");
  var storageService = Components.classes["@mozilla.org/storage/service;1"].getService(Ci.mozIStorageService);
  var connection = storageService.openDatabase(file);

  function getDataDirectory() {
    var file, prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.simplemail.");
    
    try {
      file = prefs.getComplexValue("dataDirectory", Components.interfaces.nsIRelativeFilePref).file;
    } catch(e) {
      file = prefs.getComplexValue("dataDirectory", Components.interfaces.nsIFile);
    }
 
    if (!file.exists()) 
      file.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0777);
    
    return file;
  }

  var text = "text: ";

  function bindString(statement, id, value) {
    if (value && !isNaN(Number(value))) value = text + value;
    statement.bindStringParameter(id, value);
  }

  function getString(statement, id) {
    var value = statement.getString(id);
    return value && value.substr(0, text.length) == text ? value.substr(text.length) : value;
  }

  // userok: async
  this.findAddresses = function(callbackDone, text) {
    var namemail  = text.replace(/\s+|$/g, function($0) { return "%" + $0; });    
    var statement = connection.createAsyncStatement("SELECT id, name, email FROM addressBook WHERE name LIKE :row_namemail OR email LIKE :row_namemail ORDER BY name, email");
    var addresses = [];
    
    //console.error("SM.autocomplete_findAddresses", text, namemail);//userok_log
    statement.bindByName("row_namemail", namemail);
    statement.executeAsync({
        handleResult: function(aResultSet) {
          for (var row = aResultSet.getNextRow(); row; row = aResultSet.getNextRow())
            addresses.push(new Address(row.getResultByName("id"), row.getResultByName("name"), row.getResultByName("email")));
          //console.error("SM.autocomplete_findAddressesResult", addresses);//userok_log
        },
        handleError: function(aError) {
          //console.error("SM.autocomplete_findAddresses_Error: ", aError.message, aError, connection, this);//userok_log
        },
        handleCompletion: function(aReason) {
          //if (aReason != Components.interfaces.mozIStorageStatementCallback.REASON_FINISHED)
            //console.error("SM.autocomplete_findAddresses_Query canceled or aborted!", connection, this)//userok_log
          //else {
            // call callback done
            if (callbackDone)
              callbackDone(addresses);
            // userok_log
            //console.error("SM.autocomplete_findAddresses_OK", text, namemail, addresses);
          //}
        }
    });
  }

  // userok: async
  this.deleteAddress = function(id) {
    var statement = connection.createAsyncStatement("DELETE FROM addressBook WHERE id=:row_id");
    statement.bindByName("row_id", id);
    statement.executeAsync({
      handleResult: function(aResultSet) {
      },
      handleError: function(aError) {
        //console.error("SM.autocomplete_deleteAddress_Error: ", aError.message, aError, connection, this);//userok_log
      },
      handleCompletion: function(aReason) {
        //if (aReason != Components.interfaces.mozIStorageStatementCallback.REASON_FINISHED)
        //  console.error("SM.autocomplete_deleteAddress_Query canceled or aborted!", connection, this);//userok_log
        //else console.error("SM.autocomplete_deleteAddress_OK", connection);//userok_log
      }
    });
  }
}

// Implements nsIAutoCompleteSearch
function SimpleAutoCompleteSearch() {
  this.storage = new Storage();
}

SimpleAutoCompleteSearch.prototype = {
  classID:          CLASS_ID,
  classDescription: CLASS_NAME,
  contractID:       CONTRACT_ID,

  /* userok: async
   * Search for a given string and notify a listener (either synchronously or asynchronously) of the result
   *
   * @param searchString - The string to search for
   * @param searchParam - An extra parameter
   * @param previousResult - A previous result to use for faster searchinig
   * @param listener - A listener to notify when the search is complete
   */
  startSearch: function(searchString, searchParam, result, listener) {
    var _this = this;
    this.storage.findAddresses(function(addresses) {
      //console.error("SM.autocomplete_startSearch", searchString, searchParam, addresses, this, _this);//userok_log
      var newResult = new SimpleAutoCompleteResult(searchString, Ci.nsIAutoCompleteResult.RESULT_SUCCESS, 0, "", addresses, _this.storage);
      listener.onSearchResult(_this, newResult);    
    }, searchParam);
  },

  /*
   * Stop an asynchronous search that is in progress
   */
  stopSearch: function() {
  },

  QueryInterface: XPCOMUtils.generateQI([ Ci.nsIAutoCompleteSearch ])
};

// The following line is what XPCOM uses to create components
var NSGetFactory = XPCOMUtils.generateNSGetFactory([ SimpleAutoCompleteSearch ]);